@@ -183,10 +183,8 @@ static void imap_flush(mailmessage * msg_info)
 
 static void imap_check(mailmessage * msg_info)
 {
-  int r;
-  
   if (msg_info->msg_flags != NULL) {
-    r = mail_flags_store_set(get_session_data(msg_info)->imap_flags_store,
+    mail_flags_store_set(get_session_data(msg_info)->imap_flags_store,
         msg_info);
     /* ignore errors */
   }
