@@ -639,16 +639,12 @@ int mailimf_header_string_write_driver(int (* do_write)(void *, const char *, si
   int state;
   const char * p;
   const char * word_begin;
-  const char * word_end;
-  const char * next_word;
   int first;
   
   state = STATE_BEGIN;
   
   p = str;
   word_begin = p;
-  word_end = p;
-  next_word = p;
   first = 1;
   
   while (length > 0) {
@@ -1773,10 +1769,12 @@ static int mailimf_path_write_driver(int (* do_write)(void *, const char *, size
   if (r != MAILIMF_NO_ERROR)
     return r;
 
-  r = mailimf_string_write_driver(do_write, data, col, path->pt_addr_spec,
-      strlen(path->pt_addr_spec));
-  if (r != MAILIMF_NO_ERROR)
-    return r;
+  if (path->pt_addr_spec != NULL) {
+    r = mailimf_string_write_driver(do_write, data, col, path->pt_addr_spec,
+        strlen(path->pt_addr_spec));
+    if (r != MAILIMF_NO_ERROR)
+      return r;
+  }
 
   r = mailimf_string_write_driver(do_write, data, col, ">", 1);
   if (r != MAILIMF_NO_ERROR)
