@@ -522,7 +522,6 @@ static char * write_fixed_message(char * str,
 				  const char * message, size_t size,
 				  uint32_t uid, int force_no_uid)
 {
-  size_t fixed_size;
   size_t cur_token;
   size_t left;
   int end;
@@ -532,8 +531,6 @@ static char * write_fixed_message(char * str,
 
   cur_token = 0;
 
-  fixed_size = 0;
-
   /* headers */
 
   end = FALSE;
@@ -615,7 +612,6 @@ mailmbox_append_message_list_no_lock(struct mailmbox_folder * folder,
   char * str;
   unsigned int i;
   size_t from_size;
-  size_t maxuid;
   size_t left;
   size_t crlf_count;
 
@@ -633,8 +629,6 @@ mailmbox_append_message_list_no_lock(struct mailmbox_folder * folder,
     from_size = strftime(from_line, MAX_FROM_LINE_SIZE, "From - %c\n", &time_info);
 #endif
 
-  maxuid = /* */ folder->mb_max_uid;
-
   extra_size = 0;
   for(i = 0 ; i < carray_count(append_tab) ; i ++) {
     struct mailmbox_append_info * info;
