@@ -196,7 +196,12 @@ int mailsmtp_connect(mailsmtp * session, mailstream * s)
     session->stream = NULL;
     mailstream_close(s);
     return MAILSMTP_ERROR_SERVICE_NOT_AVAILABLE;
-      
+
+  case 0:
+    session->stream = NULL;
+    mailstream_close(s);
+    return MAILSMTP_ERROR_STREAM;
+
   default:
     session->stream = NULL;
     mailstream_close(s);
@@ -271,7 +276,7 @@ static int get_hostname(mailsmtp * session, int useip, char * buf, int len)
     if (r != 0)
       return MAILSMTP_ERROR_HOSTNAME;
 
-#if (defined __linux__ || defined WIN32)
+#if (defined __linux__ || defined WIN32 || defined __sun)
     r = getnameinfo(&addr, sizeof(addr), hostname, HOSTNAME_SIZE, NULL, 0, NI_NUMERICHOST);
 #else
     r = getnameinfo(&addr, addr.sa_len, hostname, HOSTNAME_SIZE, NULL, 0, NI_NUMERICHOST);
@@ -397,6 +402,9 @@ int mailsmtp_data(mailsmtp * session)
   case 503:
     return MAILSMTP_ERROR_BAD_SEQUENCE_OF_COMMAND;
 
+  case 0:
+    return MAILSMTP_ERROR_STREAM;
+
   default:
     return MAILSMTP_ERROR_UNEXPECTED_CODE;
   }
@@ -440,45 +448,112 @@ int mailsmtp_data_message(mailsmtp * session,
   }
 }
 
+int mailsmtp_status(int smtpstatus)
+{
+  switch(smtpstatus) {
+  case 250:
+    return MAILSMTP_NO_ERROR;
+
+  case 552:
+    return MAILSMTP_ERROR_EXCEED_STORAGE_ALLOCATION;
+
+  case 554:
+    return MAILSMTP_ERROR_TRANSACTION_FAILED;
+
+  case 451:
+    return MAILSMTP_ERROR_IN_PROCESSING;
+
+  case 452:
+    return MAILSMTP_ERROR_INSUFFICIENT_SYSTEM_STORAGE;
+
+  case 0:
+    return MAILSMTP_ERROR_STREAM;
+
+  default:
+    return MAILSMTP_ERROR_UNEXPECTED_CODE;
+  }
+
+}
+
+/* lmtp data command
+ * on the end of data transaction lmtp returns a status for
+ * every recipient it delivered.
+ * recipient_list holds the rcpts from smtp_rcpt()
+ * the last error is returned to the caller
+ * if retcodes is not NULL, the individual rcpt-stati are stored there
+ *  */
+int maillmtp_data_message(mailsmtp * session,
+                          const char * message,
+                          size_t size,
+                          clist * recipient_list,
+                          int * retcodes)
+{
+  int r, ret = MAILSMTP_NO_ERROR;
+  unsigned int i = 0;
+  clistiter *iter;
+
+  r = send_data(session, message, size);
+
+  if (r == -1)
+    return MAILSMTP_ERROR_STREAM;
+
+  for (iter = clist_begin(recipient_list); iter; iter = clist_next(iter)) {
+    r = read_response(session);
+    if (MAILSMTP_NO_ERROR != mailsmtp_status(r))
+      ret = mailsmtp_status(r);
+    if (retcodes)
+      retcodes[i++] = r;
+  }
+  return ret;
+}
+
 int mailsmtp_data_message_quit(mailsmtp * session,
                                const char * message,
                                size_t size)
 {
-    int r;
+  int r = mailsmtp_data_message_quit_no_disconnect(session, message, size);
+  
+  mailstream_close(session->stream);
+  session->stream = NULL;
     
-    r = send_data(session, message, size);
-    if (r == -1)
-        return MAILSMTP_ERROR_STREAM;
-    
-    r = send_quit(session);
-    
-    r = read_response(session);
-    
-    mailstream_close(session->stream);
-    session->stream = NULL;
-    
-    switch(r) {
-        case 250:
-            return MAILSMTP_NO_ERROR;
-            
-        case 552:
-            return MAILSMTP_ERROR_EXCEED_STORAGE_ALLOCATION;
-            
-        case 554:
-            return MAILSMTP_ERROR_TRANSACTION_FAILED;
-            
-        case 451:
-            return MAILSMTP_ERROR_IN_PROCESSING;
-            
-        case 452:
-            return MAILSMTP_ERROR_INSUFFICIENT_SYSTEM_STORAGE;
-            
-        case 0:
-            return MAILSMTP_ERROR_STREAM;
-            
-        default:
-            return MAILSMTP_ERROR_UNEXPECTED_CODE;
-    }
+  return r;
+}
+
+int mailsmtp_data_message_quit_no_disconnect(mailsmtp * session,
+                                             const char * message,
+                                             size_t size)
+{
+  int r;
+  
+  r = send_data(session, message, size);
+  if (r == -1)
+    return MAILSMTP_ERROR_STREAM;
+  
+  r = send_quit(session);
+  r = read_response(session);
+  
+  switch(r) {
+    case 250:
+      return MAILSMTP_NO_ERROR;
+      
+    case 552:
+      return MAILSMTP_ERROR_EXCEED_STORAGE_ALLOCATION;
+      
+    case 554:
+      return MAILSMTP_ERROR_TRANSACTION_FAILED;
+      
+    case 451:
+      return MAILSMTP_ERROR_IN_PROCESSING;
+      
+    case 452:
+      return MAILSMTP_ERROR_INSUFFICIENT_SYSTEM_STORAGE;
+      
+    case 0:
+      return MAILSMTP_ERROR_STREAM;
+      
+    default:
+      return MAILSMTP_ERROR_UNEXPECTED_CODE;
+  }
 }
 
 /* esmtp operations */
@@ -657,6 +732,39 @@ int mailesmtp_ehlo_with_ip(mailsmtp * session, int useip)
   }
 }
 
+/* lmtp processing */
+int mailesmtp_lhlo(mailsmtp * session , const char *hostname)
+{
+  int r;
+  char command[SMTP_STRING_SIZE];
+
+  if (!hostname)
+      hostname = "localhost";
+
+  snprintf(command, SMTP_STRING_SIZE, "LHLO %s\r\n", hostname);
+  r = send_command(session, command);
+  if (r == -1)
+    return MAILSMTP_ERROR_STREAM;
+  r = read_response(session);
+
+  switch (r) {
+  case 250:
+    return mailesmtp_parse_ehlo(session);
+
+  case 504:
+    return MAILSMTP_ERROR_NOT_IMPLEMENTED;
+
+  case 550:
+    return MAILSMTP_ERROR_ACTION_NOT_TAKEN;
+
+  case 0:
+    return MAILSMTP_ERROR_STREAM;
+
+  default:
+    return MAILSMTP_ERROR_UNEXPECTED_CODE;
+  }
+}
+
 /*
   if return_full is TRUE, the entire message is returned on error
   envid can be NULL
@@ -978,6 +1086,9 @@ int mailesmtp_starttls(mailsmtp * session)
   case 454:
     return MAILSMTP_ERROR_STARTTLS_TEMPORARY_FAILURE;
 
+  case 0:
+    return MAILSMTP_ERROR_STREAM;
+
   default:
     return MAILSMTP_ERROR_UNEXPECTED_CODE;
   }
@@ -1300,6 +1411,10 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
   while (1) {
     r = read_response(session);
     switch (r) {
+    case 0:
+      res = MAILSMTP_ERROR_STREAM;
+      goto free_sasl_conn;
+        
     case 220:
     case 235:
       res = MAILSMTP_NO_ERROR;
@@ -1483,6 +1598,11 @@ int mailsmtp_send_command(mailsmtp * f, char * command)
   return send_command(f, command);
 }
 
+int mailsmtp_send_command_private(mailsmtp * f, char * command)
+{
+    return send_command_private(f, command, 0);
+}
+
 int mailsmtp_read_response(mailsmtp * session)
 {
   return read_response(session);
